@@ -720,47 +720,55 @@ DEFINE_EVENT(fs_str, bucket_alloc_fail,
 );
 
 DECLARE_EVENT_CLASS(discard_buckets_class,
-	TP_PROTO(struct bch_fs *c, u64 seen, u64 open,
-		 u64 need_journal_commit, u64 discarded, const char *err),
-	TP_ARGS(c, seen, open, need_journal_commit, discarded, err),
+	TP_PROTO(struct bch_fs *c, struct discard_buckets_state *s, const char *err),
+	TP_ARGS(c, s, err),
 
 	TP_STRUCT__entry(
 		__field(dev_t,		dev			)
 		__field(u64,		seen			)
 		__field(u64,		open			)
 		__field(u64,		need_journal_commit	)
+		__field(u64,		commit_in_flight	)
+		__field(u64,		bad_data_type		)
+		__field(u64,		already_discarding	)
 		__field(u64,		discarded		)
 		__array(char,		err,	16		)
 	),
 
 	TP_fast_assign(
 		__entry->dev			= c->dev;
-		__entry->seen			= seen;
-		__entry->open			= open;
-		__entry->need_journal_commit	= need_journal_commit;
-		__entry->discarded		= discarded;
+		__entry->seen			= s->seen;
+		__entry->open			= s->open;
+		__entry->need_journal_commit	= s->need_journal_commit;
+		__entry->commit_in_flight	= s->commit_in_flight;
+		__entry->bad_data_type		= s->bad_data_type;
+		__entry->already_discarding	= s->already_discarding;
+		__entry->discarded		= s->discarded;
 		strscpy(__entry->err, err, sizeof(__entry->err));
 	),
 
-	TP_printk("%d%d seen %llu open %llu need_journal_commit %llu discarded %llu err %s",
+	TP_printk("%d%d seen %llu open %llu\n"
+		  "need_commit %llu committing %llu bad_data_type %llu\n"
+		  "already_discarding %llu discarded %llu err %s",
 		  MAJOR(__entry->dev), MINOR(__entry->dev),
 		  __entry->seen,
 		  __entry->open,
 		  __entry->need_journal_commit,
+		  __entry->commit_in_flight,
+		  __entry->bad_data_type,
+		  __entry->already_discarding,
 		  __entry->discarded,
 		  __entry->err)
 );
 
 DEFINE_EVENT(discard_buckets_class, discard_buckets,
-	TP_PROTO(struct bch_fs *c, u64 seen, u64 open,
-		 u64 need_journal_commit, u64 discarded, const char *err),
-	TP_ARGS(c, seen, open, need_journal_commit, discarded, err)
+	TP_PROTO(struct bch_fs *c, struct discard_buckets_state *s, const char *err),
+	TP_ARGS(c, s, err)
 );
 
 DEFINE_EVENT(discard_buckets_class, discard_buckets_fast,
-	TP_PROTO(struct bch_fs *c, u64 seen, u64 open,
-		 u64 need_journal_commit, u64 discarded, const char *err),
-	TP_ARGS(c, seen, open, need_journal_commit, discarded, err)
+	TP_PROTO(struct bch_fs *c, struct discard_buckets_state *s, const char *err),
+	TP_ARGS(c, s, err)
 );
 
 TRACE_EVENT(bucket_invalidate,
@@ -1179,6 +1187,11 @@ DEFINE_EVENT(transaction_event,	trans_restart_write_buffer_flush,
 	TP_ARGS(trans, caller_ip)
 );
 
+DEFINE_EVENT(fs_str, accounting_key_to_wb_slowpath,
+	TP_PROTO(struct bch_fs *c, const char *str),
+	TP_ARGS(c, str)
+);
+
 TRACE_EVENT(path_downgrade,
 	TP_PROTO(struct btree_trans *trans,
 		 unsigned long caller_ip,
@@ -1233,25 +1246,23 @@ TRACE_EVENT(key_cache_fill,
 );
 
 TRACE_EVENT(write_buffer_flush,
-	TP_PROTO(struct btree_trans *trans, size_t nr, size_t skipped, size_t fast, size_t size),
-	TP_ARGS(trans, nr, skipped, fast, size),
+	TP_PROTO(struct btree_trans *trans, size_t nr, size_t skipped, size_t fast),
+	TP_ARGS(trans, nr, skipped, fast),
 
 	TP_STRUCT__entry(
 		__field(size_t,		nr		)
 		__field(size_t,		skipped		)
 		__field(size_t,		fast		)
-		__field(size_t,		size		)
 	),
 
 	TP_fast_assign(
 		__entry->nr	= nr;
 		__entry->skipped = skipped;
 		__entry->fast	= fast;
-		__entry->size	= size;
 	),
 
-	TP_printk("%zu/%zu skipped %zu fast %zu",
-		  __entry->nr, __entry->size, __entry->skipped, __entry->fast)
+	TP_printk("flushed %zu skipped %zu fast %zu",
+		  __entry->nr, __entry->skipped, __entry->fast)
 );
 
 TRACE_EVENT(write_buffer_flush_sync,
